"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const execa_1 = tslib_1.__importDefault(require("execa"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const path_1 = tslib_1.__importDefault(require("path"));
const constants_1 = require("../../constants");
const common_1 = require("../../common");
class ReposInstall extends command_1.Command {
    async run() {
        const { flags } = this.parse(ReposInstall);
        const resolvedBranch = flags.branch || (await this.resolveBranch());
        cli_ux_1.default.action.start('Fetching repository list');
        const selectedRepos = await common_1.getRepositories(flags.repositories ? flags.repositories.split(',') : []);
        cli_ux_1.default.action.stop(`Found ${selectedRepos.length} repos`);
        const tasks = new listr_1.default([
            {
                title: `Cloning repositories into ${flags.repos_path}`,
                task: () => new listr_1.default(selectedRepos.map(repo => ({
                    title: `${repo.name} (${resolvedBranch === 'dev' ? repo.dev : repo.stable})`,
                    task: () => this.cloneRepository(repo, resolvedBranch, flags.repos_path)
                })))
            }
        ]);
        if (flags.link_framework) {
            const linkTask = {
                title: `Linking framework (${flags.framework_path}) to repositories`,
                task: () => new listr_1.default(selectedRepos.map(repo => ({
                    title: repo.name,
                    task: () => this.linkFrameworkToRepo(repo, flags.repos_path, flags.framework_path)
                })))
            };
            tasks.add(linkTask);
        }
        await tasks.run();
    }
    async cloneRepository(repo, branch, destination) {
        this.debug(`${repo.url}#${branch === 'dev' ? repo.dev : repo.stable}`);
        return execa_1.default('git', [
            'clone',
            '--branch',
            branch === 'dev' ? repo.dev : repo.stable,
            '--depth',
            '10',
            '--single-branch',
            repo.url,
            path_1.default.join(destination, repo.name)
        ]);
    }
    async linkFrameworkToRepo(repo, destination, frameworkPath) {
        return execa_1.default('ln', [
            '-s',
            path_1.default.relative(path_1.default.join(destination, repo.name, repo.doc_root || constants_1.docPathInRepo), frameworkPath),
            path_1.default.join(destination, repo.name, repo.doc_root || constants_1.docPathInRepo, constants_1.fwDirName)
        ]);
    }
    async resolveBranch() {
        const { stdout } = await execa_1.default('git', ['rev-parse', '--abbrev-ref', 'HEAD']);
        if (stdout.match(/^master|\d+-stable$/)) {
            return 'stable';
        }
        return 'dev';
    }
}
exports.default = ReposInstall;
ReposInstall.description = 'Install a list of sub-repositories (or all) to a given destination';
ReposInstall.flags = {
    help: command_1.flags.help({ char: 'h' }),
    repositories: command_1.flags.string({
        char: 'r',
        description: 'The list of repositories to install - env: $REPOSITORIES',
        default: process.env.REPOSITORIES
    }),
    repos_path: command_1.flags.string({
        char: 'd',
        description: 'The path where the repositories will be installed - env: $REPOS_PATH',
        default: process.env.REPOS_PATH || '.repos'
    }),
    branch: command_1.flags.string({
        char: 'b',
        description: 'The branch type to checkout - env: $BRANCH',
        options: ['stable', 'dev'],
        default: process.env.BRANCH
    }),
    link_framework: command_1.flags.boolean({
        description: 'Whether to link the doc framework to the repositories or not',
        default: false
    }),
    framework_path: command_1.flags.string({
        description: 'The path to the framework',
        default: '.'
    })
};
